"""
 (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
 All Rights Reserved

 This file is part of Gideon.

 Gideon is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Gideon is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Gideon; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

# $Id: $

from ZODB.PersistentMapping import PersistentMapping

from Products.Gideon.Objects.Namespace import *
from Products.Gideon.Objects.Application.PythonRepository import RepositoryApplication
from Products.Gideon.Log.Log import log

from SaxDocumentHandlers import RSS
       
from xml.sax import make_parser
import urllib

import string
from os import path

class ChannelItem(SimpleItem):

    meta_type = 'Channel Item'
        
    index_html = DTMLFile('ui/ChannelItemView', globals())
    call_view  = DTMLFile('ui/ChannelItemCallView', globals())
    
    def __init__(self, id):
        self.id = id
        self._info = PersistentMapping()

    def title(self): return self._info.get('Title','')
    def link(self): return self._info.get('Link','')    
    def description(self): return self._info.get('Description','')    
        
    def _set_item_info(self, info):
        print 'foo', info
        convert_2_str = lambda k,v,d=info: d.update({k:str(v)})        
        [convert_2_str(k,v) for k,v in info.items()]        
        self._info.update(info)
        
    def __call__(self):
        # sigh
        return self.call_view(self, self.REQUEST)
        
        
class Channel(Folder):
    
    meta_type = 'Channel'

    index_html = DTMLFile('ui/ChannelView',globals())
    channel_widget = DTMLFile('ui/ChannelWidget', globals())
    
    def __init__(self, id):
        self.id = id
        self.data = PersistentMapping()
        
    def set_source_url(self, url):
        self.data['url'] = url
        
    def refresh_from_source(self):
        """ """
        url = self.data.get('url', '')
        if not url: return

        # we save local files for debugging
        savefile = path.join(package_home(globals()), 'var', self.getId())     

        try:
            # mime contains an open fp
            filename, mime= urllib.urlretrieve(url, savefile)
            fh = open(savefile, 'r')
        except Exception, e:
            log.warning("can't retrieve channel url: %s"%str(e))
            return

        ch = RSS()
        p = make_parser()
        p.setContentHandler(ch)
        p.parse(fh)
        fh.close()
        
        info = ch.getValues()
        
        items = info.get('Items')
        del info['Items']
        
        print 'test info', info
        ## convert to str from unicode
        #e way expensive

        convert_2_str = lambda k,v,d=info: d.update({k:str(v)})
        [convert_2_str(k,v) for k,v in info.items()]
        print 'test info', info
        
        self.data.update(info)
        
        # we just blast away old items...
        self.manage_delObjects(self.objectIds('Channel Item'))
        
        for i in xrange(len(items)):
            ci = ChannelItem(str(i))
            if not items[i].get('Link'): print 'nope' ; continue
            try: ci._set_item_info(items[i])
            except: continue 
            self._setObject(str(i), ci)
   
class NewsSink(RepositoryApplication):
    """
    A Simple RSS Channel Manager
    """
    
    meta_type = 'News Sink'

    
    index_html = None
    
    __call__ = DTMLFile('ui/NewsSinkView',globals())
    
    news_widget = DTMLFile('ui/NewsSinkWidget',globals())
    
    def __init__(self, id):
        self.id = id
        
    def reset(self, REQUEST): 
        """ """
        #self.manage_delObjects(self.objectIds())
        #self._import_defaults()
        #self._set_active_defaults() see note below
        self.refresh_channels()
        
        REQUEST.RESPONSE.redirect('.')
    
    def add_channel(self, id, channel_url):
        ob = Channel(id)
        ob.set_source_url(channel_url)
        self._setObject(id, ob)
        
    def set_channel_active(self, channel_id):
        ob = self._getOb(channel_id)
        ob.active = 1

    def get_active_channels(self):
        return [ob for ob in self.objectValues('Channel') if ob.getId() in ('ZopeNews', 'FreshMeat')]
        #return [ob for ob in self.objectValues('Channel') if hasattr(ob,'active') and ob.active == 1]
    
    def remove_channel(self, channel_id):
        self.delObjects(channel_id)
    
    def get_channel(self, channel_id):
        self._getOb(channel_id)
    
    def refresh_channels(self):
        """ """
        channels = self.objectValues('Channel')
        for c in channels: 
            try: c.refresh_from_source()
            except: pass

    def _set_active_defaults(self):
        """ calling this method seems to cause some seriously bad voodoo.
        basically it causes zserver to throw exceptions with mozilla,
        which is beyond me. its magic """
        
        for ob in self.objectValues('Channel'):
            if ob.getId() in ('ZopeNews', 'PythonCookbook', 'FreshMeat'):
                ob.active = 1
            else:
                ob.active = 0
        
    def _import_defaults(self):
        
        home = package_home(globals())
        default_file = path.join(home, 'default_channels.txt')
        fh = open(default_file)
        for l in fh.xreadlines():
            l = l.strip()
            id, url = map(string.strip, l.split('='))
            self.add_channel(id, url)
            