
"""
Gideon.Objects.Packages.Package.py

The Package Object and friends

Author: Kapil Thangavelu
License: GPL
CVS: $Id: Package.py,v 1.1.1.1 2001/12/31 04:07:06 ender Exp $

"""

from OFS.Image import cookId

from Products.Gideon.Objects.Namespace import *
from Products.Gideon.Objects.Ratings.Ratings import Rating
from Products.Gideon.Log.Log import log
from Products.Gideon.PageContract.Scripts import compile
from Products.Gideon.Objects.Catalog import CatalogSchema
from Products.Gideon.Objects.Authors.Authors import Author, make_author_id

from PackageFile import PackageFile
from PackageVersion import PackageVersionContainer, PackageVersion
from PackageDependency import PackageDependency
from PackageHistory import PackageHistory
from PackageStateVariables import PackageStateVariables


""" this is hack around issues with the presentation layer"""
class Fake:
    def __call__(self): return ''
    

class Package(Folder):

    security = ClassSecurityInfo()
    
    meta_type = 'Package' 
    
    index_html = DTMLFile('ui/PackageView', globals())

    def __init__(self, 
            id='', 
            title=''
            ):

        self.id =id
        self.title = title
        self._published = 0 ## denotes whether we fire events for
                    ## version additions
        self._initialized = 0
                    
    def get_package_admin(self):
        return self._getOb('Admin')
    
    def get_version_container(self):
        return self._getOb('Versions')
    
    def get_package_info(self):
        return getattr(self, 'info')

    def get_platforms(self):
        # should iterate through version container
        return ('all')
    
    def set_package_info(self, pi):
        self._setObject('info', pi)

    def is_published(self):
        """  """
        return self._published

    def set_published(self, published):        
        
        if published and not self._published:
            self.published_initialized()
            
        self._published = published            
    
    def get_ratings(self):
        if self._published:
            return self._getOb('Ratings')
        return Fake()
        
    def get_reviews(self):
        if self._published:
            pass
        return Fake()
    
    def get_state_id(self):
        return self._state
    
    def get_history(self):
        return self._getOb('History')
    
    def get_state_vars(self):
        return self._getOb('StateVars')
    
    def get_package(self):
        return self
    
    def _initialize(self):
        
        self._state = 'uploaded'
        self._setObject('Versions', PackageVersionContainer('Versions'))
        self._setObject('Admin', PackageAdmin('Admin'))		
        self._setObject('History', PackageHistory('History'))
        self._setObject('StateVars', PackageStateVariables('StateVars'))
                        
        self._initialized = 1

    def published_initialized(self): 
        """ """
        self._setObject('Ratings', Rating('Ratings'))
        #*# reviews
        #self._setObject('Ratings', Rating('Ratings'))		
        self._published = 1
        
    def manage_afterAdd(self, item, container):
        if not self._initialized:
            self._initialize()
        
            
            
class PackageFolder(Folder):		
    meta_type = 'Package Folder'
    index_html = DTMLFile('ui/PackageFolderView', globals())
    
    search_results = DTMLFile('ui/PackageFolderResults',globals())
    
    def search_packages(self, REQUEST):
        """ """
        contract = """
        #parameters
        SearchableText:optional,trim,nohtml
        package_licenses:optional,trim,nohtml
        """        
        contract = compile(contract)
        contract.validate(self)

        query = CatalogSchema.extract_query(REQUEST, package_schema)
        
        # temporary hacks
        query['meta_type']='Package Info'        
        if REQUEST.has_key('SearchableText'):
            query['package_description'] = REQUEST['SearchableText']
        print query

        catalog = self.get_inner_application(self).get_service("PackageCatalog")        
        
        catalog = self.get_service("PackageCatalog")
        res = catalog(**query)
        
        print res
        return res
        
        
    
        
class PackageAdmin(SimpleItem, Implicit):
    
    """
    adminstrative interface, this should
    contain most of the state dependent behavior, 
    its a kludge and i know it.
    
    ideally this class gives state and role dependent views
    to manage a package. it is mainly a presentation class
    although it contains partial logic for state changes.
    
    this would be much cleaner split up into a full state machine.
    which is what PackageWorkflow.py has the start of.
    """
    
    index_html = DTMLFile('ui/PackageAdminView', globals())
    
    edit_package_info_form = DTMLFile('ui/PackageAdminEditInfoForm',globals())
    edit_package_versions_form = DTMLFile('ui/PackageAdminEditVersionsForm', globals())
    edit_package_dependencies_form = DTMLFile('ui/PackageAdminEditDependenciesForm', globals())
    edit_package_approve_form = DTMLFile('ui/PackageAdminApproveForm', globals())
    edit_package_categories_form = DTMLFile('ui/PackageAdminEditCategories', globals())
    edit_package_authors_form = DTMLFile('ui/PackageAdminEditAuthors', globals())

    view_package_history = DTMLFile('ui/PackageHistoryView', globals())    
    
    _widget_for_queue = DTMLFile('ui/PackageAdminQueueWidget',globals())
    _widget_for_published = DTMLFile('ui/PackageAdminPublishedWidget',globals())
    _menu_for_published = DTMLFile('ui/PackageAdminPublishedMenu',globals())
    
    def __init__(self,
                 id = 'Admin'
            ):
        self.id = id
        self._state = ''
        
    def get_admin_widget(self):
        """this gets called from the package view """
        package = self.aq_inner.aq_parent
        if package._published:
            return self._widget_for_published(self, self.REQUEST)
        else:
            return self._widget_for_queue(self, self.REQUEST)
        

    def get_admin_menu(self):
        """ this gets called from the admin view """
        package = self.aq_inner.aq_parent        
        if package._published:
            return self._menu_for_published(self, self.REQUEST)
        else:
            return self._widget_for_queue(self, self.REQUEST)            
    
        
    # submit methods
        
    def edit_package_approve(self, REQUEST):
        """ """
        contract = """
        #parameters
        package_notes:optional, trim, nohtml
        """
        contract = compile(contract)
        contract.validate(self)
        
        #if REQUEST.has_key('package_notes'):
        #    package_notes = REQUEST['package_notes']
         
        pid = self.get_package_info().package_name()
        log.debug('approving package (%s) in queue'%pid)

        queue = self.get_inner_application(self)
        package = aq_parent(aq_inner(self))

        log.debug('removing package (%s) in queue'%pid)		
        queue.remove_package(package)
        
        log.debug('adding package (%s) to published packages'%pid)		
        package_container = self.get_inner_application(self, 2) 
        package_container.add_package(self.aq_parent.aq_base)
        
        # record for posterity
        history = package.get_history()
        # this is none if anonymous
        log.debug('adding note user currently %s'%getSecurityManager().getUser().getId())
        history.add_note('transition', 'package published %s'%REQUEST['package_notes'])

        # redirect back to queue
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(queue.absolute_url()+'?message='+pid+'+published')
        
    def package_add_version_file(self, REQUEST):
        """ """
        contract = """
        #parameters
        version_file:file
        file_type:trim, constraint(supported_file_types)
        platform:trim, constraint(supported_platforms)
        """
        #version_id:trim, obid(Versions)
        
        contract = compile(contract)
        
        # hmm.. err handling seems to trigger a bug... todo list for the moment
        errors = contract.validate(self, no_error_handling=1)
        if errors:
            return errors
        
        container = self.get_version_container()
        version = container._getOb(REQUEST['version_id'])
        
        id, title = cookId('','',REQUEST['version_file'])
        
        fobj= PackageFile(id, 
                          title, 
                          REQUEST['file_type'],
                          'tgz',
                          REQUEST['platform'])
        
        version.add_file(fobj)
        
        fobj = version._getOb(id)
        fobj.manage_upload(REQUEST['version_file'])
        
        history = self.aq_inner.aq_parent.get_history()
        history.add_note("addition", "file added")
        
        REQUEST.RESPONSE.redirect('.')
        
    def package_create_version(self, REQUEST):
        """ create a new package version """
        contract = """
        #parameters
        version_id:trim
        """ 
        contract = compile(contract)
        contract.validate(self)
        
        version = PackageVersion(REQUEST['version_id'])
        version_container = self.get_version_container()        
        
        version_container.add_version(version)

        history = self.aq_inner.aq_parent.get_history()
        history.add_note("addition", "version added")        

        # hmm.. where to redirect        
        REQUEST.RESPONSE.redirect('.')
                                       

    def edit_package_versions(self, REQUEST):
        """ """
        
    def package_add_dependency(self, REQUEST):
        """ """
        
    def package_add_authors(self, first_name, last_name, public_email, homepage='',
                            REQUEST=None):
        """ 
        authors for new packages are first stored as state vars
        till the package gets published, then they get added to the 
        author container via the event channel. there after they
        get added directly to the author_container.
        """
        contract = """
        first_name:trim
        last_name:trim
        public_email:trim, email
        homepage:trim
        """
        contract = compile(contract)
        contract.validate(REQUEST)

        aid = make_author_id(first_name, last_name)
        
        if self.get_package().is_published() == 1:
            author_container = self.get_application_for_name('Authors')
            if author_container.get_author(aid, None) is None:
                author_container.add_author(first_name, last_name, public_email, homepage)
        else:
            statevars = self.get_state_vars()
            authors = statevars.get('authors', [])
            authors.append(Author(aid, first_name, last_name, public_email))
            statevars['authors']=authors
            
        REQUEST.RESPONSE.redirect('.')
        
        
        
    def edit_package_info(self, REQUEST):
        """ """
        contract = """
        #parameters
        summary:trim
        license:trim, constraint(supported_licenses)
        description:trim
        platform:constraint(supported_platforms)
        """
        contract = compile(contract)
        contract.validate(self)

        vars = contract.extract_variables(REQUEST)
        info = self.get_package_info()
        
        for k,v in vars.items():
            method = getattr(info, 'set_'+k, None)
            if method: method(v)
        
        history = self.aq_inner.aq_parent.get_history()
        history.add_note("edit", "package info edited")        
        REQUEST.RESPONSE.redirect('.')

    
    def edit_package_categories(self, REQUEST):
        """ """
        # need a constrainted list contract
        contract = """
        #parameters
        package_categories:trim
        """
        categories = REQUEST['package_categories']
        info = self.get_package_info()
        
        info.set_categories(categories)
        
        # event broadcast to listeners
        if self.get_package_info().is_published():        
            channel = self.get_service("EventChannel")
            channel.publish(self.aq_inner.aq_parent, "package categories modified")
        
        history = self.aq_inner.aq_parent.get_history()
        history.add_note("edit", "package categories edited")
        REQUEST.RESPONSE.redirect('.')
        
        
        