from AccessControl import ClassSecurityInfo
from Globals import DTMLFile, InitializeClass
from Acquisition import aq_base, Implicit
from OFS.SimpleItem import SimpleItem
from OFS.Folder import Folder
from Interface import Base
from ComputedAttribute import ComputedAttribute

from Products.Gideon.Objects.Application.PythonRepository import RepositoryApplication
from Products.Gideon.Objects.Catalog import CatalogSchema
from Products.Gideon.PageContract.Scripts import compile
from Products.Gideon.Log.Log import log
from Products.Gideon import FileSystem
from Products.Gideon import MetaDataParsers

package_indexes = {
    'package_name' :'FieldIndex',
    'package_description' : 'TextIndex',
    'package_summary':'TextIndex',
    'package_url' : 'FieldIndex',
    'package_authors':'KeywordIndex',
    'package_license':'FieldIndex',
    'package_company':'FieldIndex',
    'package_keywords':'KeywordIndex',
    'package_categories':'KeywordIndex',
    'package_maintainers':'KeywordIndex',
    'is_published':'FieldIndex',		
    'uploading_user':'FieldIndex',              
    'SearchableText':'TextIndex'                                  
}

package_columns = [
    'package_name',
    'package_summary',
    'package_url',
    'package_authors',
    'package_company',
    'package_keywords',
    'package_categories'
]

package_schema = CatalogSchema.CatalogSchema(package_indexes,package_columns)

class PackageAppEventProxy(SimpleItem, Implicit): 
    
    def __init__(self, id):
        self.id = id
        
    def categories_changed(self, package, *args):
        catalog = self.get_service("PackageCatalog")
        catalog.catalog_object(package.get_package_info())
    
    def event_package_published(self, package, *args):
        catalog = self.get_service("PackageCatalog")
        catalog.catalog_object(package.get_package_info())

    def manage_afterAdd(self, item, container):
        ec = self.get_service("EventChannel")
        ec.register_listener(self, 'event_package_published', 'package published')
        ec.register_listener(self, 'categories_changed', 'package categories modified')
        
class IPackageApplication(Base): pass

class PackageApplication(RepositoryApplication,Folder):
    """
    this is a doc string.
    """
    
    meta_type = 'Package Container'
    
    security = ClassSecurityInfo()
    
    security.declarePublic('index_html')
    index_html = DTMLFile('ui/PackageContainerView', globals())
    
    security.declarePublic('search_results')
    search_results = DTMLFile('ui/PackageContainerSearchResults',globals())

    ## todo plus results page
    #security.declarePublic('advanced_search')
    #advanced_search = DTMLFile('ui/PackageContainerAdvancedSearchForm',globals())
    
    security.declarePublic('recent_packages')
    recent_packages = DTMLFile('ui/PackageContainerRecentPackages', globals())

    ## variable configuration below
    def _get_available_categories(self):
        return [c.getId() for c in self.Categories.get_all_categories()]

    supported_python_versions = ('2.2.1','2.2.0','2.1.2','2.1.1','2.1.0','2.0.2','2.0.1','2.0.0','1.6.1','1.6.0','1.5.2')
    major_python_versions = ('2.2','2.1','2.0','1.6','1.5')
    supported_platforms = ('all','win32', 'solaris', 'linux', 'mac x')
    supported_file_types = ('source', 'binary', 'documentation')
    supported_licenses = ('BSD', 'LGPL', 'GPL', 'ZPL', 'MIT', 'MPL', 'Other')
    supported_dependency_types = ('python', 'external')
    supported_dependency_kinds = ('REQUIRES', 'SUGGESTS','RECOMMENDS','CONFLICTS','REPLACES')   
    supported_architectures = ('i386', 'alpha', 'mips', 'powerpc')
    supported_file_formats = ComputedAttribute(FileSystem.get_file_formats)
    supported_metadata_formats = ComputedAttribute(MetaDataParsers.supported_formats)
    available_categories = ComputedAttribute(_get_available_categories)
    
    def get_package_queue(self):
        return self.PackageQueue
    
    def add_package(self, package):
        # play name games
        log.debug('publishing package')
        pid = package.get_package_info().package_name()
        char = pid.lower()[0]

        # store it
        package_folder = self._getOb(char, None)
        if package_folder is None:
            package_folder = self._getOb('misc')
        package_folder._setObject(pid, aq_base(package))
        
        # get a newly wrapped reference
        package = package_folder._getOb(pid)

        # tell the package its approved
        package.set_published(1)
        
        # index it
        #log.debug('publishing package - indexing')        
        #cat = self.get_service("PackageCatalog")
        #cat.catalog_object(package.get_package_info())
        
        # tell others about it
        ec = self.get_service("EventChannel")
        ec.publish(package, 'package published')
        
    security.declarePublic("search_packages")
    def search_packages(self, REQUEST):
        contract = """
        #parameters
        SearchableText:optional,trim,nohtml
        package_licenses:optional,trim,nohtml
        """        
        contract = compile(contract)
        contract.validate(self)

        query = CatalogSchema.extract_query(REQUEST, package_schema)
        
        # temporary hacks
        query['meta_type']='Package Info'        
        if REQUEST.has_key('SearchableText'):
            query['SearchableText'] = REQUEST['SearchableText']

        catalog = self.get_service("PackageCatalog")
        res = catalog(**query)
        
        return res

    security.declarePublic('get_package_licenses')
    def get_package_licenses(self):
        catalog = self.get_service("PackageCatalog")
        return catalog.uniqueValuesFor('package_license')
    
    security.declarePrivate('get_package_filesystem')
    def get_package_filesystem(self):
        return get_file_system()
    
    security.declarePublic('get_packages_for_author')
    def get_packages_for_author(self, author):
        catalog = self.get_service("PackageCatalog")
        query = {}
        query['package_authors']=author
        query['meta_type']='Package Info'
        res = catalog(**query)
        return res
    
    security.declarePublic('get_related_packages')
    def get_related_packages(self, package):
        """ Hmm.. how to define relationships, categories? """
        cat = self.get_site_catalog()
        return cat.query()
    
    def manage_afterAdd(self, item, container):
        """ """
        CatalogSchema.install_catalog_schema('PackageCatalog', package_schema, self)
        self._setObject('event_proxy', PackageAppEventProxy('event_proxy'))
    
InitializeClass(PackageApplication)	