from AccessControl import ClassSecurityInfo, getSecurityManager
from OFS.Folder import Folder
from Globals import DTMLFile, InitializeClass


from Products.Gideon.Objects.Catalog import CatalogSchema
from Products.Gideon.Objects.Application.PythonRepository import RepositoryApplication
from Products.Gideon.Objects.Namespace import *
from Products.Gideon.Log.Log import log

package_indexes = {

    'package_name' :'FieldIndex',
    'package_description' : 'TextIndex',
    'package_summary':'TextIndex',
    'package_url' : 'FieldIndex',
    'package_authors':'KeywordIndex',
    'package_company':'FieldIndex', 
	'package_keywords':'KeywordIndex',
	'package_categories':'KeywordIndex',
	'assigned_user':'FieldIndex',
	'is_published':'FieldIndex',
	'uploading_user':'FieldIndex'
    
}

package_columns = [
    'package_name',
    'package_summary',
    'package_url',
    'package_authors',
    'package_company',
    'package_keywords',
    'package_categories'
]

queue_schema = CatalogSchema.CatalogSchema(package_indexes,package_columns)


class PackageQueueEventProxy(SimpleItem,  Implicit):
	"""
	this class handles events for the queue
	"""
	
	def __init__(self, id):
		self.id = id
		
	def notifications_to_groups(self):
		pass
	
	def unindex_package(self, package, *args):
		cat = self.get_inner_application(self).get_service('QueueCatalog')
		log.debug('unindexing %s'%'/'.join(package.get_package_info().getPhysicalPath()))		
		cat.uncatalog_object('/'.join(package.get_package_info().getPhysicalPath()))
	
	def index_package(self, package, *args):
		print 'hello'
		cat = self.get_inner_application(self).get_service('QueueCatalog')
		cat.catalog_object(package.get_package_info())
		
	def manage_afterAdd(self, item, container):
		log.debug('registering package queue w/ ec')
		ec = self.get_service("EventChannel")
		ec.register_listener(self, 'index_package', 'package added to queue')
		ec.register_listener(self, 'unindex_package', 'package removed from queue')		
		
class PackageQueue(RepositoryApplication):
	"""
	Notes
	
  	need to query a catalog for info regarding queue size

	"""
	_isApplication = 1

	meta_type = 'Package Queue'
	
	security = ClassSecurityInfo()	

	index_html = DTMLFile('ui/PackageQueueView', globals())
	standard_html_header = DTMLFile('ui/PackageQueueHeader', globals())
	results_html = DTMLFile('ui/PackageQueueSearchResults', globals())
	
	def __init__(self, id):
		self.id = id

	def add_package(self, pkg_id, package):
		""" add a package and broadcast its inclusion
		to listeners """

		self._setObject(pkg_id, aq_base(package))
		package = self._getOb(pkg_id)
		channel = self.get_service("EventChannel")		
		channel.publish(package, 'package added to queue')
		
		package.get_history().add_note('transition', 'package added to queue')
		
	def reindex_all(self):
		""" """
		catalog = self.QueueCatalog
		
		for o in self.objectValues('Package'):
			catalog.catalog_object(o.get_package_info())
			
	
	def remove_package(self, package):
		# package should have proper aq wrapper
		channel = self.get_service("EventChannel")		
		channel.publish(package, 'package removed from queue')
		self._delObject(package.get_package_info().package_name())
		
	def get_user_assigned_packages(self):
		catalog = self.get_service("QueueCatalog")
		userid = getSecurityManager().getUser().getId()
		print userid
		res = catalog(meta_type='Package Info', assigned_user=userid)
		return res
		
	def get_unassigned_packages(self):
		cat = self.get_service("QueueCatalog")
		res = cat(meta_type='Package Info', assigned_user=('',))
		# this is prolly too expensive for large result sets
		# although the cat does some batching internally...
		return [cat.getobject(r.data_record_id_) for r in res]
	
	def assign_package(self, pkgid, REQUEST):
		""" FIX allow user assignment """
		
		pkg = self._getOb(pkgid)
		userid = getSecurityManager().getUser().getId()
		info = pkg.get_package_info()
		info.set_assigned_user(userid)
		catalog = self.QueueCatalog
		print 'cataloging'
		catalog.catalog_object(info)
		
		REQUEST.RESPONSE.redirect('.')
		
	def search_queue_packages(self, REQUEST):
		""" """
		catalog = self.get_service("QueueCatalog")
		res = catalog(meta_type='Package Info', SearchableText=REQUEST.get('SearchableText'))
		return self.results_html(self, REQUEST, results=res)
			
	def manage_afterAdd(self, item, container):
		CatalogSchema.install_catalog_schema('QueueCatalog', queue_schema, self)
		self._setObject('EventProxy', PackageQueueEventProxy('EventProxy'))
