"""
 (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
 All Rights Reserved

 This file is part of Gideon.

 Gideon is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Gideon is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Gideon; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

# $Id: $

from ZODB.PersistentMapping import PersistentMapping

from Products.Gideon.Objects.Namespace import *
from Products.Gideon.Log.Log import log
from Products.Gideon.Objects.Application.PythonRepository import RepositoryApplication

class IServiceBroker(Base):
    """
    The Service Broker acts as container and lookup registry
    for web services 
    """
        
    def get_adaptor(self, protocol_name):
        """ return a protocol adaptor for a given protocol name """
    
    def get_provider(self, method_name):
        """ get a provider for the given method name """
    
    def register_provider(self, provider, provider_methods):
        """ register a provider for the given methods """
    
    def register_adaptor(self, adaptor, protocol_name):
        """ register a protocol adaptor """

class ServiceBroker(ObjectManager):
    
    # ideally this would overide bobo traverse to allow
    # for adaptors to live non contained, but as is
    # thats not needed yet, so the current implementation
    # lets only providers live non local
    
    meta_type = 'Service Broker'
    
    __implements__ = (IServiceBroker,)

    security = ClassSecurityInfo()
    
    def __init__(self, id):
        self.id = id
        self.protocols = PersistentMapping()
        self.methods   = PersistentMapping()
    
    security.declarePrivate('register_provider')
    def register_provider(self, provider, provider_methods):
        
        provider_path = '/'.join(provider.getPhysicalPath())
        
        for m in provider_methods:
            
            if self.methods.has_key(m):
                log.warning('Service Broker : overiding method registeration')
            
            self.methods[m]=provider_path
        
            
    security.declarePrivate('register_adaptor')                
    def register_adaptor(self, adaptor, protocol_name):
        
        adaptor_path = '/'.join(adaptor.getPhysicalPath())
        
        if self.protocols.has_key(protocol_name):
            log.warning('Service Broker: overiding protocol registeration')
            
        self.protocols[protocol_name]=adaptor_path
        
    def get_provider_info(self):
        return self.methods.items()    
    
    def get_adaptor_info(self):
        return self.protocols.items()
        
    def get_provider(self, method_name):
        
        provider_path = self.methods.get(method_name)
        return self.unrestrictedTraverse(provider_path)
    
    def get_adaptor(self, protocol_name):

        adaptor_path = self.protocols.get(protocol_name)
        return self.unrestrictedTraverse(protocol_name)
    
        
class RepositoryServiceBroker(RepositoryApplication, ServiceBroker): 
    
    index_html = DTMLFile('ui/RepositoryBrokerView',globals())
    
    def _install_defaults(self):
        #import os
        #configpath = os.sep.join(package_home(globals()), 'provider_config')
        #fh = open(configpath)
        
        from ServiceAdaptor import XMLRPCServiceAdaptor
        ob = XMLRPCServiceAdaptor('xmlrpc')
        self._setObject('xmlrpc', ob)
        
        from ServiceProvider import RepositoryServiceProvider
        ob = RepositoryServiceProvider('provider')
        self._setObject('provider', ob)
        
        
    
    
    
        
        
        